/* 

***Read this first***

This SAS program imports one or more QCEW flat files into
one SAS data set and stores it as a SAS file in a 
user-specified folder.

The program will accept any of the "singlefile" versions
of the beta files, or the old *.enb or *.end formats. In order
for the program to work correctly, do not mix format
types. Note that the resulting database may take up
several hundred megabytes or even several gigabytes.
Make sure you have enough hard drive or network drive
space for the resulting files.

To use this program, follow these instructions:

1) Go to the QCEW FTP site:
ftp://ftp.bls.gov/pub/special.requests/cew/

If you want to use the beta files, open the "/beta" directory.

2) Open the folder for the year you need.

3)  For the traditional flat files, download the file ending in
.all.enb.zip, select the level of geographic aggregation that you
are interested in, and extract the file for the area you want data for. 

For the beta files, download the annual or quarterly .zip file that ends in
"singlefile.zip". Open the .zip file, and extract the enclosed file to a folder that
your SAS program can access. This is the source folder.

4) If you want to import multiple files, repeat steps 2 through 5
until you have all the files you need.

5) Type or copy/paste the source folder path and file name of the first file
between the parentheses in the statement below. Here is an example:
%let sourceFile1 = %str(C:\SAS\data\2010.q1-q4.singlefile.csv);
*/

%let sourceFile1 = %str();

/* If you are importing just one file, skip to the next step.
If you want to import multiple files, copy one line of the code below
for each file you want to import, and paste it in the white space
outside the comment tags. Put the complete folder path and file name of
each file in its own set of parentheses. Make sure that consecutive
sourceFile numbers are used. The program won't work if there are gaps
in the numbering. If you are importing more files than this, you can
write more lines of code by increasing the number at the end of
the sourceFile variable by 1 for each new line of code.*/
/*
%let sourceFile2 = %str();
%let sourceFile3 = %str();
%let sourceFile4 = %str();
%let sourceFile5 = %str();
%let sourceFile6 = %str();
%let sourceFile7 = %str();
%let sourceFile8 = %str();
%let sourceFile9 = %str();
%let sourceFile10 = %str();

Paste the code you want to use in the white space here:
*/






/*

6) How many files are you importing? Put that number between the
parentheses below. For example, if you are importing 4 files,
the code looks like this:
%let nmax = %str(4);
*/
%let nmax = %str();
/*

7) Decide what folder should contain the SAS file itself, and
type or paste that folder path in the parentheses below: */

%let outPath = %str();


/*

8) Run the SAS code.

Notes:
If you need information on what the area, ownership, industry, and other
codes mean, see this folder on our FTP site:
ftp://ftp.bls.gov/pub/special.requests/cew/beta/titles/

For other questions and assistance, please call 202-691-6567 or submit
an information request at http://data.bls.gov/cgi-bin/forms/cew?/cew/cewcont.htm.

This SAS program was written in SAS 9.1.3 SP4 for Windows.

/* ---------------------------------- */

libname qcewData "&outPath";

%macro annual_singlefile_import(qcewSourceFile=);
data qcewData.qcewSingleFile;
	infile "&qcewSourceFile"
		dsd /*comma delimited*/ firstobs=2 /*remove header*/;
	/*colon gives the delimiter priority over the number of positions to read*/
	input 	area_fips :$5.
			own :$1.
			naics :$6.
			agglvl_code :$2.
			size_code :$1.
			year :$4.
			qtr :$1.
			disclosure_code :$1.
			annual_avg_estabs :8.
			annual_avg_emplvl :9.
			total_annual_wages :15.
			taxable_annual_wages :15.
			annual_contributions :15.
			annual_avg_wkly_wage :8.
			avg_annual_pay :8.
			;

run;

proc append base=qcewData.qcewFile data=qcewData.qcewSingleFile;

%mend annual_singlefile_import;

%macro quarterly_singlefile_import(qcewSourceFile=);
data qcewData.qcewSingleFile;
	infile "&qcewSourceFile"
		dsd /*comma delimited*/ firstobs=2 /*remove header*/;
	/*colon gives the delimiter priority over the number of positions to read*/
	input 	area_fips :$5.
			own :$1.
			naics :$6.
			agglvl_code :$2.
			size_code :$1.
			year :$4.
			qtr :$1.
			disclosure_code :$1.
			qtrly_estabs :8.
			month1_emplvl :9.
			month2_emplvl :9.
			month3_emplvl :9.
			total_qtrly_wages :15.
			taxable_qtrly_wages :15.
			qtrly_contributions :15.
			avg_wkly_wage :8.
			;

run;

proc append base=qcewData.qcewFile data=qcewData.qcewSingleFile;

%mend quarterly_singlefile_import;

%macro enb_file_import(qcewSourceFile=);
data qcewData.qcewSingleFile;
          /**********************************************************************************
          *Purpose: read in CEW Format ENB records
          *Version Number: 1.0
          *Date of Last Update: 01/06/03
          ***********************************************************************************/
    infile "&qcewSourceFile" lrecl=449;
       input

          /****Identifier Fields****/
          @1    survpref   $3.  /*Survey Prefix*/
          @4    area       $5.  /*Area Code*/
          @9    datatype    1.  /*Datatype Code*/
          @10   size        1.  /*Size Code*/
          @11   own         1.  /*Ownership*/
          @12   naics      $6.  /*NAICS Industry Code*/
          @18   year        4.  /*Year*/
          @22   agglvl      2.  /*Aggregation Level*/

          /****First Quarter Fields****/
          @24   statq1     $1.  /*Status-Disclosure Code*/
          @25   estabq1     8.  /*Number of Establishments*/
          @33   m1          9.  /*January Employment*/
          @42   m2          9.  /*February Employment*/
          @51   m3          9.  /*March Employment*/
          @60   wageq1     15.  /*Total Quarterly Wages*/
          @75   txwageq1   15.  /*Taxable Quarterly Wages*/
          @90   cntrbq1    13.  /*Quarterly Contributions*/
          @103  awwq1       8.  /*Average Weekly Wage*/

          /****Second Quarter Fields****/
          @111  statq2     $1.  /*Status-Disclosure Code*/
          @112  estabq2     8.  /*Quarterly Number of Establishments*/
          @120  m4          9.  /*April Employment*/
          @129  m5          9.  /*May Employment*/
          @138  m6          9.  /*June Employment*/
          @147  wageq2     15.  /*Total Quarterly Wages*/
          @162  txwageq2   15.  /*Taxable Quarterly Wages*/
          @177  cntrbq2    13.  /*Quarterly Contributions*/
          @190  awwq2       8.  /*Average Weekly Wage*/

          /****Third Quarter Fields****/
          @198  statq3     $1.  /*Status-Disclosure Code*/
          @199  estabq3     8.  /*Quarterly Number of Establishments*/
          @207  m7          9.  /*July Employment*/
          @216  m8          9.  /*August Employment*/
          @225  m9          9.  /*September Employment*/
          @234  wageq3     15.  /*Total Quarterly Wages*/
          @249  txwageq3   15.  /*Taxable Quarterly Wages*/
          @264  cntrbq3    13.  /*Quarterly Contributions*/
          @277  awwq3       8.  /*Average Weekly Wage*/

          /****Fourth Quarter Fields****/
          @285  statq4     $1.  /*Status-Disclosure Code*/
          @286  estabq4     8.  /*Quarterly Number of Establishments*/
          @294  m10         9.  /*October Employment*/
          @303  m11         9.  /*November Employment*/
          @312  m12         9.  /*December Employment*/
          @321  wageq4     15.  /*Total Quarterly Wages*/
          @336  txwageq4   15.  /*Taxable Quarterly Wages*/
          @351  cntrbq4    13.  /*Quarterly Contributions*/
          @364  awwq4       8.  /*Average Weekly Wage*/

          /****Annual Fields****/
          @372  annstat    $1.  /*Status-Disclosure Code*/
          @373  annestab    8.  /*Annual Average Number of Establishments*/
          @381  annemp      9.  /*Annual Average Employment*/
          @390  annwage    15.  /*Total Annual Wages*/
          @405  antxwage   15.  /*Taxable Annual Wages*/
          @420  anncntrb   13.  /*Annual Contributions*/
          @433  annaww      8.  /*Average Weekly Wage*/
          @441  aap         9.  /*Average Annual Pay*/;
run;

proc append base=qcewData.qcewFile data=qcewData.qcewSingleFile;

%mend enb_file_import;

%macro end_file_import(qcewSourceFile=);
data qcewData.qcewSingleFile;
          /**********************************************************************************
          *Purpose: read in CEW Format single quarter END records
          *Version Number: 1.0
          *Date of Last Update: 04/15/04
          ***********************************************************************************/
    infile "&qcewSourceFile" lrecl=111;
       input

          /****Identifier Fields****/
          @1    survpref   $3.  /*Survey Prefix*/
          @4    area       $5.  /*Area Code*/
          @9    datatype    1.  /*Datatype Code*/
          @10   size        1.  /*Size Code*/
          @11   own         1.  /*Ownership*/
          @12   naics      $6.  /*NAICS Industry Code*/
          @18   year        4.  /*Year*/
          @22   quarter     1.  /*Quarter*/
          @23   agglvl      2.  /*Aggregation Level*/

          /****Data Fields****/
          @25   status     $1.  /*Status-Disclosure Code*/
          @26   estab       8.  /*Number of Establishments*/
          @34   m1emp       9.  /*Month 1 Employment*/
          @43   m2emp       9.  /*Month 2 Employment*/
          @52   m3emp       9.  /*Month 3 Employment*/
          @61   wage       15.  /*Total Quarterly Wages*/
          @76   txwage     15.  /*Taxable Quarterly Wages*/
          @91   cntrb      13.  /*Quarterly Contributions*/
          @104  aww         8.  /*Average Weekly Wage*/;
run;

proc append base=qcewData.qcewFile data=qcewData.qcewSingleFile;

%mend end_file_import;


%macro makesas;
%do n=1 %to &nmax;
	%let sourceFileLength&n = %length(&&sourceFile&n);
	%if %qsubstr(&&sourcefile&n,&&sourceFileLength&n - 20,17) = %str(annual.singlefile) %then
	%annual_singlefile_import(qcewSourceFile=%str(&&sourcefile&n));
	%else %if %qsubstr(&&sourcefile&n,&&sourceFileLength&n - 16,13) = %str(q1.singlefile) or
	    %qsubstr(&&sourcefile&n,&&sourceFileLength&n - 19,16) = %str(q1-q2.singlefile) or
	    %qsubstr(&&sourcefile&n,&&sourceFileLength&n - 19,16) = %str(q1-q3.singlefile) or
	    %qsubstr(&&sourcefile&n,&&sourceFileLength&n - 19,16) = %str(q1-q4.singlefile) %then
		%quarterly_singlefile_import(qcewSourceFile=%str(&&sourcefile&n));
	%else %if %qsubstr(&&sourcefile&n,&&sourceFileLength&n - 3,4) = %str(.end) %then
		%end_file_import(qcewSourceFile=%str(&&sourcefile&n));
	%else %if %qsubstr(&&sourcefile&n,&&sourceFileLength&n - 3,4) = %str(.enb) %then
		%enb_file_import(qcewSourceFile=%str(&&sourcefile&n));
	%else %put The file names submitted are not recognized by this program.;
%end;
%mend makesas;



%makesas;
proc datasets library=qcewData;
delete qcewSingleFile;
run;
quit;

